@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,18 +16,30 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#include <cmath>
-#include <cstring>
-#include <iostream>
 
+// prototype/interface header file
+#include "ogg_decoder.hpp"
+
+// local headers
 #include "common/aixlog.hpp"
 #include "common/endian.hpp"
 #include "common/snap_exception.hpp"
-#include "ogg_decoder.hpp"
+
+// 3rd party headers
+
+// standard headers
+#ifndef HAS_TREMOR
+#include <cmath> // floor
+#endif
+#include <cstdint>
+#include <cstring>
+
 
 
 using namespace std;
 
+static constexpr auto LOG_TAG = "OggDecoder";
+
 namespace decoder
 {
 
@@ -73,12 +85,12 @@ bool OggDecoder::decode(msg::PcmChunk* chunk)
         if (result < 0)
         {
             /* missing or corrupt data at this page position */
-            LOG(ERROR) << "Corrupt or missing data in bitstream; continuing...\n";
+            LOG(ERROR, LOG_TAG) << "Corrupt or missing data in bitstream; continuing...\n";
             continue;
         }
 
         ogg_stream_pagein(&os, &og); /* can safely ignore errors at this point */
-        while (1)
+        while (true)
         {
             result = ogg_stream_packetout(&os, &op);
 
@@ -104,46 +116,46 @@ bool OggDecoder::decode(msg::PcmChunk* chunk)
             (-1.<=range<=1.) to whatever PCM format and write it out */
             while ((samples = vorbis_synthesis_pcmout(&vd, &pcm)) > 0)
             {
-                size_t bytes = sampleFormat_.sampleSize() * vi.channels * samples;
-                chunk->payload = (char*)realloc(chunk->payload, chunk->payloadSize + bytes);
+                uint32_t bytes = sampleFormat_.sampleSize() * vi.channels * samples;
+                chunk->payload = static_cast<char*>(realloc(chunk->payload, chunk->payloadSize + bytes));
                 for (int channel = 0; channel < vi.channels; ++channel)
                 {
                     if (sampleFormat_.sampleSize() == 1)
                     {
-                        int8_t* chunkBuffer = (int8_t*)(chunk->payload + chunk->payloadSize);
+                        auto* chunkBuffer = reinterpret_cast<int8_t*>(chunk->payload + chunk->payloadSize);
                         for (int i = 0; i < samples; i++)
                         {
                             int8_t& val = chunkBuffer[sampleFormat_.channels() * i + channel];
 #ifdef HAS_TREMOR
-                            val = clip<int8_t>(pcm[channel][i], -128, 127);
+                            val = clip<int8_t>(pcm[channel][i], INT8_MIN, INT8_MAX);
 #else
-                            val = clip<int8_t>(floor(pcm[channel][i] * 127.f + .5f), -128, 127);
+                            val = clip<int8_t>(floor(pcm[channel][i] * 127.f + .5f), INT8_MIN, INT8_MAX);
 #endif
                         }
                     }
                     else if (sampleFormat_.sampleSize() == 2)
                     {
-                        int16_t* chunkBuffer = (int16_t*)(chunk->payload + chunk->payloadSize);
+                        auto* chunkBuffer = reinterpret_cast<int16_t*>(chunk->payload + chunk->payloadSize);
                         for (int i = 0; i < samples; i++)
                         {
                             int16_t& val = chunkBuffer[sampleFormat_.channels() * i + channel];
 #ifdef HAS_TREMOR
-                            val = SWAP_16(clip<int16_t>(pcm[channel][i] >> 9, -32768, 32767));
+                            val = SWAP_16(clip<int16_t>(pcm[channel][i] >> 9, INT16_MIN, INT16_MAX));
 #else
-                            val = SWAP_16(clip<int16_t>(floor(pcm[channel][i] * 32767.f + .5f), -32768, 32767));
+                            val = SWAP_16(clip<int16_t>(floor(pcm[channel][i] * 32767.f + .5f), INT16_MIN, INT16_MAX));
 #endif
                         }
                     }
                     else if (sampleFormat_.sampleSize() == 4)
                     {
-                        int32_t* chunkBuffer = (int32_t*)(chunk->payload + chunk->payloadSize);
+                        auto* chunkBuffer = reinterpret_cast<int32_t*>(chunk->payload + chunk->payloadSize);
                         for (int i = 0; i < samples; i++)
                         {
                             int32_t& val = chunkBuffer[sampleFormat_.channels() * i + channel];
 #ifdef HAS_TREMOR
-                            val = SWAP_32(clip<int32_t>(pcm[channel][i] << 7, -2147483648, 2147483647));
+                            val = SWAP_32(clip<int32_t>(pcm[channel][i] << 7, INT32_MIN, INT32_MAX));
 #else
-                            val = SWAP_32(clip<int32_t>(floor(pcm[channel][i] * 2147483647.f + .5f), -2147483648, 2147483647));
+                            val = SWAP_32(clip<int32_t>(floor(pcm[channel][i] * 2147483647.f + .5f), INT32_MIN, INT32_MAX));
 #endif
                         }
                     }
@@ -222,21 +234,21 @@ SampleFormat OggDecoder::setHeader(msg::CodecHeader* chunk)
     /// local state for most of the decode so multiple block decodes can proceed
     /// in parallel. We could init multiple vorbis_block structures for vd here
 
-    sampleFormat_.setFormat(vi.rate, 16, vi.channels);
+    sampleFormat_.setFormat(vi.rate, 16, static_cast<uint16_t>(vi.channels));
 
     /* Throw the comments plus a few lines about the bitstream we're decoding */
     char** ptr = vc.user_comments;
-    while (*ptr)
+    while (*ptr != nullptr)
     {
         std::string comment(*ptr);
         if (comment.find("SAMPLE_FORMAT=") == 0)
-            sampleFormat_.setFormat(comment.substr(comment.find("=") + 1));
-        LOG(INFO) << "comment: " << comment << "\n";
+            sampleFormat_.setFormat(comment.substr(comment.find('=') + 1));
+        LOG(INFO, LOG_TAG) << "comment: " << comment << "\n";
         ;
         ++ptr;
     }
 
-    LOG(INFO) << "Encoded by: " << vc.vendor << "\n";
+    LOG(INFO, LOG_TAG) << "Encoded by: " << vc.vendor << "\n";
 
     return sampleFormat_;
 }
