@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,18 +16,20 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
+// prototype/interface header file
 #include "daemon.hpp"
 
+// local headers
 #include "common/snap_exception.hpp"
 #include "common/str_compat.hpp"
-#include "common/utils.hpp"
 #include "common/utils/file_utils.hpp"
+
+// standard headers
+#include <cstdlib>
+#include <cstring>
 #include <fcntl.h>
 #include <grp.h>
-#include <iostream>
 #include <pwd.h>
-#include <stdlib.h>
-#include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <syslog.h>
@@ -62,12 +64,12 @@ void Daemon::daemonize()
         throw SnapException("Could not open PID lock file \"" + pidfile_ + "\"");
     }
 
-    uid_t user_uid = (uid_t)-1;
-    gid_t user_gid = (gid_t)-1;
-    std::string user_name;
-    //#ifdef FREEBSD
-    //	bool had_group = false;
-    //#endif
+    auto user_uid = static_cast<uid_t>(-1);
+    auto user_gid = static_cast<gid_t>(-1);
+    // std::string user_name;
+    // #ifdef FREEBSD
+    //     bool had_group = false;
+    // #endif
 
     if (!user_.empty())
     {
@@ -76,9 +78,9 @@ void Daemon::daemonize()
             throw SnapException("no such user \"" + user_ + "\"");
         user_uid = pwd->pw_uid;
         user_gid = pwd->pw_gid;
-        user_name = strdup(user_.c_str());
+        // user_name = strdup(user_.c_str());
         /// this is needed by libs such as arts
-        setenv("HOME", pwd->pw_dir, true);
+        setenv("HOME", pwd->pw_dir, 1);
     }
 
     if (!group_.empty())
@@ -87,9 +89,9 @@ void Daemon::daemonize()
         if (grp == nullptr)
             throw SnapException("no such group \"" + group_ + "\"");
         user_gid = grp->gr_gid;
-        //#ifdef FREEBSD
-        //		had_group = true;
-        //#endif
+        // #ifdef FREEBSD
+        //         had_group = true;
+        // #endif
     }
 
     if (chown(pidfile_.c_str(), user_uid, user_gid) == -1)
@@ -99,19 +101,19 @@ void Daemon::daemonize()
     }
 
     /// set gid
-    if (user_gid != (gid_t)-1 && user_gid != getgid() && setgid(user_gid) == -1)
-        throw SnapException("Failed to set group " + cpt::to_string((int)user_gid));
+    if (user_gid != static_cast<gid_t>(-1) && user_gid != getgid() && setgid(user_gid) == -1)
+        throw SnapException("Failed to set group " + cpt::to_string(static_cast<int>(user_gid)));
 
-    //#if defined(FREEBSD) && !defined(MACOS)
-    //#ifdef FREEBSD
+    // #if defined(FREEBSD) && !defined(MACOS)
+    // #ifdef FREEBSD
     /// init supplementary groups
     /// (must be done before we change our uid)
     /// no need to set the new user's supplementary groups if we are already this user
     //	if (!had_group && user_uid != getuid() && initgroups(user_name, user_gid) == -1)
     //		throw SnapException("Failed to set supplementary groups of user \"" + user + "\"");
-    //#endif
+    // #endif
     /// set uid
-    if (user_uid != (uid_t)-1 && user_uid != getuid() && setuid(user_uid) == -1)
+    if (user_uid != static_cast<uid_t>(-1) && user_uid != getuid() && setuid(user_uid) == -1)
         throw SnapException("Failed to set user " + user_);
 
     /// Our process ID and Session ID
@@ -155,7 +157,7 @@ void Daemon::daemonize()
     sprintf(str, "%d\n", getpid());
 
     /// write pid to lockfile
-    if (write(pidFilehandle_, str, strlen(str)) != (int)strlen(str))
+    if (write(pidFilehandle_, str, strlen(str)) != static_cast<int>(strlen(str)))
         throw SnapException("Could not write PID to lock file \"" + pidfile_ + "\"");
 
     /// Close out the standard file descriptors
