@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,17 +16,21 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#ifndef MESSAGE_H
-#define MESSAGE_H
+#pragma once
 
+// local headers
 #include "common/endian.hpp"
 #include "common/time_defs.hpp"
+
+// standard headers
 #include <cstdlib>
 #include <cstring>
 #include <iostream>
 #include <streambuf>
+#ifndef WINDOWS
 #include <sys/time.h>
-#include <vector>
+#endif
+#include <memory>
 
 /*
 template<typename CharT, typename TraitsT = std::char_traits<CharT> >
@@ -49,7 +53,7 @@ struct membuf : public std::basic_streambuf<char>
 };
 
 
-enum message_type
+enum class message_type : uint16_t
 {
     kBase = 0,
     kCodecHeader = 1,
@@ -57,12 +61,43 @@ enum message_type
     kServerSettings = 3,
     kTime = 4,
     kHello = 5,
-    kStreamTags = 6,
+    // kStreamTags = 6,
+    kClientInfo = 7,
 
     kFirst = kBase,
-    kLast = kStreamTags
+    kLast = kClientInfo
 };
 
+static std::ostream& operator<<(std::ostream& os, const message_type& msg_type)
+{
+    switch (msg_type)
+    {
+        case message_type::kBase:
+            os << "Base";
+            break;
+        case message_type::kCodecHeader:
+            os << "CodecHeader";
+            break;
+        case message_type::kWireChunk:
+            os << "WireChunk";
+            break;
+        case message_type::kServerSettings:
+            os << "ServerSettings";
+            break;
+        case message_type::kTime:
+            os << "Time";
+            break;
+        case message_type::kHello:
+            os << "Hello";
+            break;
+        case message_type::kClientInfo:
+            os << "ClientInfo";
+            break;
+        default:
+            os << "Unknown";
+    }
+    return os;
+}
 
 struct tv
 {
@@ -117,11 +152,11 @@ using message_ptr = std::shared_ptr<msg::BaseMessage>;
 
 struct BaseMessage
 {
-    BaseMessage() : type(kBase), id(0), refersTo(0)
+    BaseMessage() : BaseMessage(message_type::kBase)
     {
     }
 
-    BaseMessage(message_type type_) : type(type_), id(0), refersTo(0)
+    BaseMessage(message_type type_) : type(type_), id(0), refersTo(0), size(0)
     {
     }
 
@@ -178,7 +213,7 @@ struct BaseMessage
         return 3 * sizeof(uint16_t) + 2 * sizeof(tv) + sizeof(uint32_t);
     };
 
-    uint16_t type;
+    message_type type;
     mutable uint16_t id;
     uint16_t refersTo;
     tv received;
@@ -203,6 +238,12 @@ protected:
         stream.write(reinterpret_cast<const char*>(&v), sizeof(uint16_t));
     }
 
+    void writeVal(std::ostream& stream, const message_type& val) const
+    {
+        uint16_t v = SWAP_16(static_cast<uint16_t>(val));
+        stream.write(reinterpret_cast<const char*>(&v), sizeof(uint16_t));
+    }
+
     void writeVal(std::ostream& stream, const int16_t& val) const
     {
         uint16_t v = SWAP_16(val);
@@ -229,8 +270,8 @@ protected:
 
     void writeVal(std::ostream& stream, const std::string& val) const
     {
-        uint32_t size = val.size();
-        writeVal(stream, val.c_str(), size);
+        auto len = static_cast<uint32_t>(val.size());
+        writeVal(stream, val.c_str(), len);
     }
 
 
@@ -250,24 +291,34 @@ protected:
     void readVal(std::istream& stream, uint16_t& val) const
     {
         stream.read(reinterpret_cast<char*>(&val), sizeof(uint16_t));
+        // cppcheck-suppress selfAssignment
         val = SWAP_16(val);
     }
 
+    void readVal(std::istream& stream, message_type& val) const
+    {
+        stream.read(reinterpret_cast<char*>(&val), sizeof(uint16_t));
+        val = static_cast<message_type>(SWAP_16(static_cast<uint16_t>(val)));
+    }
+
     void readVal(std::istream& stream, int16_t& val) const
     {
         stream.read(reinterpret_cast<char*>(&val), sizeof(int16_t));
+        // cppcheck-suppress selfAssignment
         val = SWAP_16(val);
     }
 
     void readVal(std::istream& stream, uint32_t& val) const
     {
         stream.read(reinterpret_cast<char*>(&val), sizeof(uint32_t));
+        // cppcheck-suppress selfAssignment
         val = SWAP_32(val);
     }
 
     void readVal(std::istream& stream, int32_t& val) const
     {
         stream.read(reinterpret_cast<char*>(&val), sizeof(int32_t));
+        // cppcheck-suppress selfAssignment
         val = SWAP_32(val);
     }
 
@@ -280,10 +331,10 @@ protected:
 
     void readVal(std::istream& stream, std::string& val) const
     {
-        uint32_t size;
-        readVal(stream, size);
-        val.resize(size);
-        stream.read(&val[0], size);
+        uint32_t len;
+        readVal(stream, len);
+        val.resize(len);
+        stream.read(&val[0], len);
     }
 
 
@@ -291,5 +342,3 @@ protected:
 };
 
 } // namespace msg
-
-#endif
