@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2022  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,12 +16,15 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#ifndef PCM_CHUNK_H
-#define PCM_CHUNK_H
+#ifndef MESSAGE_PCM_CHUNK_HPP
+#define MESSAGE_PCM_CHUNK_HPP
 
+// local headers
 #include "common/sample_format.hpp"
 #include "message.hpp"
 #include "wire_chunk.hpp"
+
+// standard headers
 #include <chrono>
 
 
@@ -36,7 +39,7 @@ namespace msg
 class PcmChunk : public WireChunk
 {
 public:
-    PcmChunk(const SampleFormat& sampleFormat, size_t ms)
+    PcmChunk(const SampleFormat& sampleFormat, uint32_t ms)
         : WireChunk((sampleFormat.rate() * ms / 1000) * sampleFormat.frameSize()), format(sampleFormat), idx_(0)
     {
     }
@@ -58,7 +61,20 @@ public:
     }
 #endif
 
-    int readFrames(void* outputBuffer, size_t frameCount)
+    // std::unique_ptr<PcmChunk> consume(uint32_t frameCount)
+    // {
+    //     auto result = std::make_unique<PcmChunk>(format, 0);
+    //     if (frameCount * format.frameSize() > payloadSize)
+    //         frameCount = payloadSize / format.frameSize();
+    //     result->payload = payload;
+    //     result->payloadSize = frameCount * format.frameSize();
+    //     payloadSize -= result->payloadSize;
+    //     payload = (char*)realloc(payload + result->payloadSize, payloadSize);
+    //     // payload += result->payloadSize;
+    //     return result;
+    // }
+
+    int readFrames(void* outputBuffer, uint32_t frameCount)
     {
         // logd << "read: " << frameCount << ", total: " << (wireChunk->length / format.frameSize()) << ", idx: " << idx;// << std::endl;
         int result = frameCount;
@@ -77,8 +93,8 @@ public:
 
     int seek(int frames)
     {
-        if ((frames < 0) && (-frames > (int)idx_))
-            frames = -idx_;
+        if ((frames < 0) && (-frames > static_cast<int>(idx_)))
+            frames = -static_cast<int>(idx_);
 
         idx_ += frames;
         if (idx_ > getFrameCount())
@@ -87,7 +103,6 @@ public:
         return idx_;
     }
 
-
     chronos::time_point_clk start() const override
     {
         return chronos::time_point_clk(chronos::sec(timestamp.sec) + chronos::usec(timestamp.usec) +
@@ -105,6 +120,14 @@ public:
         return std::chrono::duration_cast<T>(chronos::nsec(static_cast<chronos::nsec::rep>(1000000 * getFrameCount() / format.msRate())));
     }
 
+    // void append(const PcmChunk& chunk)
+    // {
+    //     auto newSize = payloadSize + chunk.payloadSize;
+    //     payload = (char*)realloc(payload, newSize);
+    //     memcpy(payload + payloadSize, chunk.payload, chunk.payloadSize);
+    //     payloadSize = newSize;
+    // }
+
     double durationMs() const
     {
         return static_cast<double>(getFrameCount()) / format.msRate();
@@ -121,12 +144,12 @@ public:
         return idx_ >= getFrameCount();
     }
 
-    inline size_t getFrameCount() const
+    inline uint32_t getFrameCount() const
     {
         return (payloadSize / format.frameSize());
     }
 
-    inline size_t getSampleCount() const
+    inline uint32_t getSampleCount() const
     {
         return (payloadSize / format.sampleSize());
     }
@@ -134,7 +157,7 @@ public:
     SampleFormat format;
 
 private:
-    uint32_t idx_;
+    uint32_t idx_ = 0;
 };
 } // namespace msg
 
