@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,9 +16,10 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#ifndef SAMPLE_FORMAT_H
-#define SAMPLE_FORMAT_H
+#pragma once
 
+// standard headers
+#include <cstdint>
 #include <string>
 
 
@@ -41,7 +42,7 @@ public:
     SampleFormat(const std::string& format);
     SampleFormat(uint32_t rate, uint16_t bits, uint16_t channels);
 
-    std::string getFormat() const;
+    std::string toString() const;
 
     void setFormat(const std::string& format);
     void setFormat(uint32_t rate, uint16_t bits, uint16_t channels);
@@ -100,6 +101,3 @@ private:
     uint16_t bits_;
     uint16_t channels_;
 };
-
-
-#endif
