@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,31 +16,30 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#ifndef CONTROL_SESSION_H
-#define CONTROL_SESSION_H
+#pragma once
 
-#include "common/queue.h"
-#include "message/message.hpp"
-#include "server_settings.hpp"
-#include <atomic>
-#include <boost/asio.hpp>
-#include <condition_variable>
+// local headers
+
+// 3rd party headers
+
+// standard headers
+#include <functional>
 #include <memory>
-#include <set>
 #include <string>
 
-using boost::asio::ip::tcp;
-
 
 class ControlSession;
-
+class StreamSession;
 
 /// Interface: callback for a received message.
 class ControlMessageReceiver
 {
 public:
+    using ResponseHander = std::function<void(const std::string& response)>;
     // TODO: rename, error handling
-    virtual std::string onMessageReceived(ControlSession* connection, const std::string& message) = 0;
+    virtual void onMessageReceived(std::shared_ptr<ControlSession> session, const std::string& message, const ResponseHander& response_handler) = 0;
+    virtual void onNewSession(std::shared_ptr<ControlSession> session) = 0;
+    virtual void onNewSession(std::shared_ptr<StreamSession> session) = 0;
 };
 
 
@@ -50,10 +49,10 @@ public:
  * Messages are sent to the client with the "send" method.
  * Received messages from the client are passed to the ControlMessageReceiver callback
  */
-class ControlSession
+class ControlSession : public std::enable_shared_from_this<ControlSession>
 {
 public:
-    /// ctor. Received message from the client are passed to MessageReceiver
+    /// ctor. Received message from the client are passed to ControlMessageReceiver
     ControlSession(ControlMessageReceiver* receiver) : message_receiver_(receiver)
     {
     }
@@ -61,16 +60,9 @@ public:
     virtual void start() = 0;
     virtual void stop() = 0;
 
-    /// Sends a message to the client (synchronous)
-    virtual bool send(const std::string& message) = 0;
-
     /// Sends a message to the client (asynchronous)
     virtual void sendAsync(const std::string& message) = 0;
 
 protected:
     ControlMessageReceiver* message_receiver_;
 };
-
-
-
-#endif
