@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,15 +16,62 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
+// prototype/interface header file
 #include "control_session_http.hpp"
-#include "common/aixlog.hpp"
-#include "message/pcm_chunk.hpp"
-#include <boost/beast/http/file_body.hpp>
+
+// standard headers
 #include <iostream>
 
+// 3rd party headers
+#include <boost/beast/http/buffer_body.hpp>
+#include <boost/beast/http/file_body.hpp>
+
+// local headers
+#include "common/aixlog.hpp"
+#include "common/message/pcm_chunk.hpp"
+#include "common/utils/file_utils.hpp"
+#include "control_session_ws.hpp"
+#include "stream_session_ws.hpp"
+
 using namespace std;
+namespace websocket = beast::websocket; // from <boost/beast/websocket.hpp>
+
+static constexpr auto LOG_TAG = "ControlSessionHTTP";
+
 
 static constexpr const char* HTTP_SERVER_NAME = "Snapcast";
+static constexpr const char* UNCONFIGURED =
+    "<html><head><title>Snapcast Default Page</title></head>"
+    "<body>"
+    "  <h1>Snapcast Default Page</h1>"
+    "  <p>"
+    "    This is the default welcome page used to test the correct operation of the Snapcast built-in webserver."
+    "  </p>"
+    "  <p>"
+    "    This webserver is a websocket endpoint for control clients (ws://<i>host</i>:1780/jsonrpc) and streaming clients"
+    "    (ws://<i>host</i>:1780/stream), but it can also host simple web pages. To serve a web page, you must configure the"
+    "    document root in the snapserver configuration file <b>snapserver.conf</b>, usually located in"
+    "    <b>/etc/snapserver.conf</b>"
+    "  </p>"
+    "  <p>"
+    "    The Snapserver installation should include a copy of <a href=\"https://github.com/badaix/snapweb\">Snapweb</a>,"
+    "    located in <b>/usr/share/snapserver/snapweb/</b><br>"
+    "    To activate it, please configure the <b>doc_root</b> as follows, and restart Snapserver to activate the changes:"
+    "  </p>"
+    "  <pre>"
+    "# HTTP RPC #####################################\n"
+    "#\n"
+    "[http]\n"
+    "\n"
+    "...\n"
+    "\n"
+    "# serve a website from the doc_root location\n"
+    "doc_root = /usr/share/snapserver/snapweb/\n"
+    "\n"
+    "#\n"
+    "################################################</pre>"
+    "</body>"
+    "</html>";
 
 namespace
 {
@@ -32,7 +79,8 @@ namespace
 boost::beast::string_view mime_type(boost::beast::string_view path)
 {
     using boost::beast::iequals;
-    auto const ext = [&path] {
+    auto const ext = [&path]
+    {
         auto const pos = path.rfind(".");
         if (pos == boost::beast::string_view::npos)
             return boost::beast::string_view{};
@@ -88,8 +136,8 @@ boost::beast::string_view mime_type(boost::beast::string_view path)
 std::string path_cat(boost::beast::string_view base, boost::beast::string_view path)
 {
     if (base.empty())
-        return path.to_string();
-    std::string result = base.to_string();
+        return std::string(path);
+    std::string result = std::string(base);
     char constexpr path_separator = '/';
     if (result.back() == path_separator)
         result.resize(result.size() - 1);
@@ -98,25 +146,23 @@ std::string path_cat(boost::beast::string_view base, boost::beast::string_view p
 }
 } // namespace
 
-ControlSessionHttp::ControlSessionHttp(ControlMessageReceiver* receiver, boost::asio::io_context& ioc, tcp::socket&& socket,
-                                       const ServerSettings::HttpSettings& settings)
-    : ControlSession(receiver), socket_(std::move(socket)), settings_(settings), strand_(ioc)
+ControlSessionHttp::ControlSessionHttp(ControlMessageReceiver* receiver, tcp::socket&& socket, const ServerSettings::Http& settings)
+    : ControlSession(receiver), socket_(std::move(socket)), settings_(settings)
 {
-    LOG(DEBUG) << "ControlSessionHttp\n";
+    LOG(DEBUG, LOG_TAG) << "ControlSessionHttp, Local IP: " << socket_.local_endpoint().address().to_string() << "\n";
 }
 
 
 ControlSessionHttp::~ControlSessionHttp()
 {
-    LOG(DEBUG) << "ControlSessionHttp::~ControlSessionHttp()\n";
+    LOG(DEBUG, LOG_TAG) << "ControlSessionHttp::~ControlSessionHttp()\n";
     stop();
 }
 
 
 void ControlSessionHttp::start()
 {
-    http::async_read(socket_, buffer_, req_, boost::asio::bind_executor(strand_, [ this, self = shared_from_this() ](
-                                                                                     boost::system::error_code ec, std::size_t bytes) { on_read(ec, bytes); }));
+    http::async_read(socket_, buffer_, req_, [this, self = shared_from_this()](boost::system::error_code ec, std::size_t bytes) { on_read(ec, bytes); });
 }
 
 
@@ -128,35 +174,50 @@ template <class Body, class Allocator, class Send>
 void ControlSessionHttp::handle_request(http::request<Body, http::basic_fields<Allocator>>&& req, Send&& send)
 {
     // Returns a bad request response
-    auto const bad_request = [&req](boost::beast::string_view why) {
+    auto const bad_request = [&req](boost::beast::string_view why)
+    {
         http::response<http::string_body> res{http::status::bad_request, req.version()};
         // TODO: Server: Snapcast/VERSION
         res.set(http::field::server, HTTP_SERVER_NAME);
         res.set(http::field::content_type, "text/html");
         res.keep_alive(req.keep_alive());
-        res.body() = why.to_string();
+        res.body() = std::string(why);
         res.prepare_payload();
         return res;
     };
 
     // Returns a not found response
-    auto const not_found = [&req](boost::beast::string_view target) {
+    auto const not_found = [&req](boost::beast::string_view target)
+    {
         http::response<http::string_body> res{http::status::not_found, req.version()};
         res.set(http::field::server, HTTP_SERVER_NAME);
         res.set(http::field::content_type, "text/html");
         res.keep_alive(req.keep_alive());
-        res.body() = "The resource '" + target.to_string() + "' was not found.";
+        res.body() = "The resource '" + std::string(target) + "' was not found.";
+        res.prepare_payload();
+        return res;
+    };
+
+    // Returns a configuration help
+    auto const unconfigured = [&req]()
+    {
+        http::response<http::string_body> res{http::status::ok, req.version()};
+        res.set(http::field::server, HTTP_SERVER_NAME);
+        res.set(http::field::content_type, "text/html");
+        res.keep_alive(req.keep_alive());
+        res.body() = UNCONFIGURED;
         res.prepare_payload();
         return res;
     };
 
     // Returns a server error response
-    auto const server_error = [&req](boost::beast::string_view what) {
+    auto const server_error = [&req](boost::beast::string_view what)
+    {
         http::response<http::string_body> res{http::status::internal_server_error, req.version()};
         res.set(http::field::server, HTTP_SERVER_NAME);
         res.set(http::field::content_type, "text/html");
         res.keep_alive(req.keep_alive());
-        res.body() = "An error occurred: '" + what.to_string() + "'";
+        res.body() = "An error occurred: '" + std::string(what) + "'";
         res.prepare_payload();
         return res;
     };
@@ -171,29 +232,64 @@ void ControlSessionHttp::handle_request(http::request<Body, http::basic_fields<A
         if (req.target() != "/jsonrpc")
             return send(bad_request("Illegal request-target"));
 
-        string response = message_receiver_->onMessageReceived(this, req.body());
-        http::response<http::string_body> res{http::status::ok, req.version()};
-        res.set(http::field::server, HTTP_SERVER_NAME);
-        res.set(http::field::content_type, "application/json");
-        res.keep_alive(req.keep_alive());
-        res.body() = response;
-        res.prepare_payload();
-        return send(std::move(res));
+        std::string request = req.body();
+        return message_receiver_->onMessageReceived(shared_from_this(), request,
+                                                    [req = std::move(req), send = std::move(send)](const std::string& response)
+                                                    {
+            http::response<http::string_body> res{http::status::ok, req.version()};
+            res.set(http::field::server, HTTP_SERVER_NAME);
+            res.set(http::field::content_type, "application/json");
+            res.keep_alive(req.keep_alive());
+            res.body() = response;
+            res.prepare_payload();
+            return send(std::move(res));
+        });
     }
 
     // Request path must be absolute and not contain "..".
-    if (req.target().empty() || req.target()[0] != '/' || req.target().find("..") != beast::string_view::npos)
+    auto target = req.target();
+    if (target.empty() || target[0] != '/' || target.find("..") != beast::string_view::npos)
         return send(bad_request("Illegal request-target"));
 
-    if (settings_.doc_root.empty())
+    static const string image_cache_target = "/__image_cache?name=";
+    auto pos = target.find(image_cache_target);
+    if (pos != std::string::npos)
+    {
+        pos += image_cache_target.size();
+        target = target.substr(pos);
+        auto image = settings_.image_cache.getImage(std::string(target));
+        LOG(DEBUG, LOG_TAG) << "image cache: " << target << ", found: " << image.has_value() << "\n";
+        if (image.has_value())
+        {
+            http::response<http::buffer_body> res{http::status::ok, req.version()};
+            res.body().data = image.value().data();
+            const auto size = image.value().size();
+            res.body().size = size;
+            res.body().more = false;
+            res.set(http::field::server, HTTP_SERVER_NAME);
+            res.set(http::field::content_type, mime_type(target));
+            res.content_length(size);
+            res.keep_alive(req.keep_alive());
+            return send(std::move(res));
+        }
         return send(not_found(req.target()));
+    }
 
     // Build the path to the requested file
-    std::string path = path_cat(settings_.doc_root, req.target());
+    std::string path = path_cat(settings_.doc_root, target);
     if (req.target().back() == '/')
         path.append("index.html");
 
-    LOG(DEBUG) << "path: " << path << "\n";
+    if (settings_.doc_root.empty())
+    {
+        static constexpr auto default_page = "/usr/share/snapserver/index.html";
+        if (utils::file::exists(default_page))
+            path = default_page;
+        else
+            return send(unconfigured());
+    }
+
+    LOG(DEBUG, LOG_TAG) << "path: " << path << "\n";
     // Attempt to open the file
     beast::error_code ec;
     http::file_body::value_type body;
@@ -242,26 +338,62 @@ void ControlSessionHttp::on_read(beast::error_code ec, std::size_t bytes_transfe
     // Handle the error, if any
     if (ec)
     {
-        LOG(ERROR) << "ControlSessionHttp::on_read error: " << ec.message() << "\n";
+        LOG(ERROR, LOG_TAG) << "ControlSessionHttp::on_read error: " << ec.message() << "\n";
         return;
     }
 
-    LOG(DEBUG) << "read: " << bytes_transferred << ", method: " << req_.method_string() << ", content type: " << req_[beast::http::field::content_type]
-               << ", target: " << req_.target() << ", body: " << req_.body() << "\n";
+    LOG(DEBUG, LOG_TAG) << "read: " << bytes_transferred << ", method: " << req_.method_string() << ", content type: " << req_[beast::http::field::content_type]
+                        << ", target: " << req_.target() << ", body: " << req_.body() << "\n";
 
     // See if it is a WebSocket Upgrade
-    if (websocket::is_upgrade(req_) && (req_.target() == "/jsonrpc"))
+    if (websocket::is_upgrade(req_))
     {
-        // Create a WebSocket session by transferring the socket
-        // std::make_shared<websocket_session>(std::move(socket_), state_)->run(std::move(req_));
-        ws_ = make_unique<websocket::stream<beast::tcp_stream>>(std::move(socket_));
-        ws_->async_accept(req_, [ this, self = shared_from_this() ](beast::error_code ec) { on_accept_ws(ec); });
-        LOG(DEBUG) << "websocket upgrade\n";
+        LOG(DEBUG, LOG_TAG) << "websocket upgrade, target: " << req_.target() << "\n";
+        if (req_.target() == "/jsonrpc")
+        {
+            // Create a WebSocket session by transferring the socket
+            // std::make_shared<websocket_session>(std::move(socket_), state_)->run(std::move(req_));
+            auto ws = std::make_shared<websocket::stream<beast::tcp_stream>>(std::move(socket_));
+            ws->async_accept(req_,
+                             [this, ws, self = shared_from_this()](beast::error_code ec)
+                             {
+                if (ec)
+                {
+                    LOG(ERROR, LOG_TAG) << "Error during WebSocket handshake (control): " << ec.message() << "\n";
+                }
+                else
+                {
+                    auto ws_session = make_shared<ControlSessionWebsocket>(message_receiver_, std::move(*ws));
+                    message_receiver_->onNewSession(std::move(ws_session));
+                }
+            });
+        }
+        else if (req_.target() == "/stream")
+        {
+            // Create a WebSocket session by transferring the socket
+            // std::make_shared<websocket_session>(std::move(socket_), state_)->run(std::move(req_));
+            auto ws = std::make_shared<websocket::stream<beast::tcp_stream>>(std::move(socket_));
+            ws->async_accept(req_,
+                             [this, ws, self = shared_from_this()](beast::error_code ec)
+                             {
+                if (ec)
+                {
+                    LOG(ERROR, LOG_TAG) << "Error during WebSocket handshake (stream): " << ec.message() << "\n";
+                }
+                else
+                {
+                    auto ws_session = make_shared<StreamSessionWebsocket>(nullptr, std::move(*ws));
+                    message_receiver_->onNewSession(std::move(ws_session));
+                }
+            });
+        }
         return;
     }
 
     // Send the response
-    handle_request(std::move(req_), [this](auto&& response) {
+    handle_request(std::move(req_),
+                   [this](auto&& response)
+                   {
         // The lifetime of the message has to extend
         // for the duration of the async operation so
         // we use a shared_ptr to manage it.
@@ -270,19 +402,19 @@ void ControlSessionHttp::on_read(beast::error_code ec, std::size_t bytes_transfe
 
         // Write the response
         http::async_write(this->socket_, *sp,
-                          boost::asio::bind_executor(strand_, [ this, self = this->shared_from_this(), sp ](beast::error_code ec, std::size_t bytes) {
-                              this->on_write(ec, bytes, sp->need_eof());
-                          }));
+                          [this, self = this->shared_from_this(), sp](beast::error_code ec, std::size_t bytes) { this->on_write(ec, bytes, sp->need_eof()); });
     });
 }
 
 
-void ControlSessionHttp::on_write(beast::error_code ec, std::size_t, bool close)
+void ControlSessionHttp::on_write(beast::error_code ec, std::size_t bytes, bool close)
 {
+    std::ignore = bytes;
+
     // Handle the error, if any
     if (ec)
     {
-        LOG(ERROR) << "ControlSessionHttp::on_write, error: " << ec.message() << "\n";
+        LOG(ERROR, LOG_TAG) << "ControlSessionHttp::on_write, error: " << ec.message() << "\n";
         return;
     }
 
@@ -299,8 +431,7 @@ void ControlSessionHttp::on_write(beast::error_code ec, std::size_t, bool close)
     req_ = {};
 
     // Read another request
-    http::async_read(socket_, buffer_, req_,
-                     boost::asio::bind_executor(strand_, [ this, self = shared_from_this() ](beast::error_code ec, std::size_t bytes) { on_read(ec, bytes); }));
+    http::async_read(socket_, buffer_, req_, [this, self = shared_from_this()](beast::error_code ec, std::size_t bytes) { on_read(ec, bytes); });
 }
 
 
@@ -308,103 +439,7 @@ void ControlSessionHttp::stop()
 {
 }
 
-void ControlSessionHttp::sendAsync(const std::string& message)
+
+void ControlSessionHttp::sendAsync(const std::string& /*message*/)
 {
-    if (!ws_)
-        return;
-
-    strand_.post([ this, self = shared_from_this(), message ]() {
-        messages_.emplace_back(message);
-        if (messages_.size() > 1)
-        {
-            LOG(DEBUG) << "HTTP session outstanding async_writes: " << messages_.size() << "\n";
-            return;
-        }
-        send_next();
-    });
-}
-
-void ControlSessionHttp::send_next()
-{
-    if (!ws_)
-        return;
-
-    auto message = messages_.front();
-    ws_->async_write(boost::asio::buffer(message),
-                     boost::asio::bind_executor(strand_, [ this, self = shared_from_this() ](std::error_code ec, std::size_t length) {
-                         messages_.pop_front();
-                         if (ec)
-                         {
-                             LOG(ERROR) << "Error while writing to web socket: " << ec.message() << "\n";
-                         }
-                         else
-                         {
-                             LOG(DEBUG) << "Wrote " << length << " bytes to web socket\n";
-                         }
-                         if (!messages_.empty())
-                             send_next();
-                     }));
-}
-
-
-bool ControlSessionHttp::send(const std::string& message)
-{
-    if (!ws_)
-        return false;
-
-    boost::system::error_code ec;
-    ws_->write(boost::asio::buffer(message), ec);
-    return !ec;
-}
-
-void ControlSessionHttp::on_accept_ws(beast::error_code ec)
-{
-    if (ec)
-    {
-        LOG(ERROR) << "ControlSessionWs::on_accept, error: " << ec.message() << "\n";
-        return;
-    }
-
-    // Read a message
-    do_read_ws();
-}
-
-void ControlSessionHttp::do_read_ws()
-{
-    // Read a message into our buffer
-    ws_->async_read(buffer_, boost::asio::bind_executor(strand_, [ this, self = shared_from_this() ](beast::error_code ec, std::size_t bytes_transferred) {
-                        on_read_ws(ec, bytes_transferred);
-                    }));
-}
-
-
-void ControlSessionHttp::on_read_ws(beast::error_code ec, std::size_t bytes_transferred)
-{
-    boost::ignore_unused(bytes_transferred);
-
-    // This indicates that the session was closed
-    if (ec == websocket::error::closed)
-        return;
-
-    if (ec)
-    {
-        LOG(ERROR) << "ControlSessionHttp::on_read_ws error: " << ec.message() << "\n";
-        return;
-    }
-
-    std::string line{boost::beast::buffers_to_string(buffer_.data())};
-    if (!line.empty())
-    {
-        // LOG(DEBUG) << "received: " << line << "\n";
-        if ((message_receiver_ != nullptr) && !line.empty())
-        {
-            string response = message_receiver_->onMessageReceived(this, line);
-            if (!response.empty())
-            {
-                sendAsync(response);
-            }
-        }
-    }
-    buffer_.consume(bytes_transferred);
-    do_read_ws();
 }
