@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,33 +16,22 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#ifndef ENCODER_H
-#define ENCODER_H
+#pragma once
 
+// local headers
+#include "common/message/codec_header.hpp"
+#include "common/message/pcm_chunk.hpp"
+#include "common/sample_format.hpp"
+
+// standard headers
+#include <functional>
 #include <memory>
 #include <string>
 
-#include "common/sample_format.hpp"
-#include "message/codec_header.hpp"
-#include "message/pcm_chunk.hpp"
 
 namespace encoder
 {
 
-class Encoder;
-
-/// Callback interface for users of Encoder
-/**
- * Users of Encoder should implement this to get the encoded PCM data
- */
-class EncoderListener
-{
-public:
-    virtual void onChunkEncoded(const Encoder* encoder, std::shared_ptr<msg::PcmChunk> chunk, double duration) = 0;
-};
-
-
-
 /// Abstract Encoder class
 /**
  * Stream encoder. PCM chunks are fed into the encoder.
@@ -51,6 +40,8 @@ public:
 class Encoder
 {
 public:
+    using OnEncodedCallback = std::function<void(const Encoder&, std::shared_ptr<msg::PcmChunk>, double)>;
+
     /// ctor. Codec options (E.g. compression level) are passed as string and are codec dependend
     Encoder(const std::string& codecOptions = "") : headerChunk_(nullptr), codecOptions_(codecOptions)
     {
@@ -59,17 +50,17 @@ public:
     virtual ~Encoder() = default;
 
     /// The listener will receive the encoded stream
-    virtual void init(EncoderListener* listener, const SampleFormat& format)
+    virtual void init(OnEncodedCallback callback, const SampleFormat& format)
     {
         if (codecOptions_ == "")
             codecOptions_ = getDefaultOptions();
-        listener_ = listener;
+        encoded_callback_ = callback;
         sampleFormat_ = format;
         initEncoder();
     }
 
     /// Here the work is done. Encoded data is passed to the EncoderListener.
-    virtual void encode(const msg::PcmChunk* chunk) = 0;
+    virtual void encode(const msg::PcmChunk& chunk) = 0;
 
     virtual std::string name() const = 0;
 
@@ -94,10 +85,8 @@ protected:
 
     SampleFormat sampleFormat_;
     std::shared_ptr<msg::CodecHeader> headerChunk_;
-    EncoderListener* listener_;
     std::string codecOptions_;
+    OnEncodedCallback encoded_callback_;
 };
 
 } // namespace encoder
-
-#endif
