@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,16 +16,19 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#ifndef FLAC_ENCODER_H
-#define FLAC_ENCODER_H
+#pragma once
+
+// local headers
 #include "encoder.hpp"
+
+// 3rd party headers
+#include "FLAC/stream_encoder.h"
+
+// standard headers
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "FLAC/metadata.h"
-#include "FLAC/stream_encoder.h"
-
 namespace encoder
 {
 
@@ -34,7 +37,7 @@ class FlacEncoder : public Encoder
 public:
     FlacEncoder(const std::string& codecOptions = "");
     ~FlacEncoder() override;
-    void encode(const msg::PcmChunk* chunk) override;
+    void encode(const msg::PcmChunk& chunk) override;
     std::string getAvailableOptions() const override;
     std::string getDefaultOptions() const override;
     std::string name() const override;
@@ -56,5 +59,3 @@ protected:
 };
 
 } // namespace encoder
-
-#endif
