@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,21 +16,27 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#include <cstring>
-#include <iostream>
+// prototype/interface header file
+#include "ogg_encoder.hpp"
 
+// local headers
 #include "common/aixlog.hpp"
 #include "common/snap_exception.hpp"
 #include "common/str_compat.hpp"
-#include "common/utils.hpp"
 #include "common/utils/string_utils.hpp"
-#include "ogg_encoder.hpp"
+
+// standard headers
+#include <cstring>
+#include <iostream>
+
 
 using namespace std;
 
 namespace encoder
 {
 
+static constexpr auto LOG_TAG = "OggEnc";
+
 OggEncoder::OggEncoder(const std::string& codecOptions) : Encoder(codecOptions), lastGranulepos_(0)
 {
 }
@@ -64,12 +70,13 @@ std::string OggEncoder::name() const
 }
 
 
-void OggEncoder::encode(const msg::PcmChunk* chunk)
+void OggEncoder::encode(const msg::PcmChunk& chunk)
 {
     double res = 0;
-    // LOG(TRACE) << "payload: " << chunk->payloadSize << "\tframes: " << chunk->getFrameCount() << "\tduration: " << chunk->duration<chronos::msec>().count()
+    // LOG(TRACE, LOG_TAG) << "payload: " << chunk->payloadSize << "\tframes: " << chunk->getFrameCount() << "\tduration: " <<
+    // chunk->duration<chronos::msec>().count()
     // << "\n";
-    int frames = chunk->getFrameCount();
+    int frames = chunk.getFrameCount();
     float** buffer = vorbis_analysis_buffer(&vd_, frames);
 
     /* uninterleave samples */
@@ -77,19 +84,19 @@ void OggEncoder::encode(const msg::PcmChunk* chunk)
     {
         if (sampleFormat_.sampleSize() == 1)
         {
-            int8_t* chunkBuffer = (int8_t*)chunk->payload;
+            auto* chunkBuffer = reinterpret_cast<int8_t*>(chunk.payload);
             for (int i = 0; i < frames; i++)
                 buffer[channel][i] = chunkBuffer[sampleFormat_.channels() * i + channel] / 128.f;
         }
         else if (sampleFormat_.sampleSize() == 2)
         {
-            int16_t* chunkBuffer = (int16_t*)chunk->payload;
+            auto* chunkBuffer = reinterpret_cast<int16_t*>(chunk.payload);
             for (int i = 0; i < frames; i++)
                 buffer[channel][i] = chunkBuffer[sampleFormat_.channels() * i + channel] / 32768.f;
         }
         else if (sampleFormat_.sampleSize() == 4)
         {
-            int32_t* chunkBuffer = (int32_t*)chunk->payload;
+            auto* chunkBuffer = reinterpret_cast<int32_t*>(chunk.payload);
             for (int i = 0; i < frames; i++)
                 buffer[channel][i] = chunkBuffer[sampleFormat_.channels() * i + channel] / 2147483648.f;
         }
@@ -98,7 +105,7 @@ void OggEncoder::encode(const msg::PcmChunk* chunk)
     /* tell the library how much we actually submitted */
     vorbis_analysis_wrote(&vd_, frames);
 
-    auto oggChunk = make_shared<msg::PcmChunk>(chunk->format, 0);
+    auto oggChunk = make_shared<msg::PcmChunk>(chunk.format, 0);
 
     /* vorbis does some data preanalysis, then divvies up blocks for
     more involved (potentially parallel) processing.  Get a single
@@ -110,7 +117,7 @@ void OggEncoder::encode(const msg::PcmChunk* chunk)
         vorbis_analysis(&vb_, nullptr);
         vorbis_bitrate_addblock(&vb_);
 
-        while (vorbis_bitrate_flushpacket(&vd_, &op_))
+        while (vorbis_bitrate_flushpacket(&vd_, &op_) != 0)
         {
             /* weld the packet into the bitstream */
             ogg_stream_packetin(&os_, &op_);
@@ -126,14 +133,14 @@ void OggEncoder::encode(const msg::PcmChunk* chunk)
                 size_t nextLen = pos + og_.header_len + og_.body_len;
                 // make chunk larger
                 if (oggChunk->payloadSize < nextLen)
-                    oggChunk->payload = (char*)realloc(oggChunk->payload, nextLen);
+                    oggChunk->payload = static_cast<char*>(realloc(oggChunk->payload, nextLen));
 
                 memcpy(oggChunk->payload + pos, og_.header, og_.header_len);
                 pos += og_.header_len;
                 memcpy(oggChunk->payload + pos, og_.body, og_.body_len);
                 pos += og_.body_len;
 
-                if (ogg_page_eos(&og_))
+                if (ogg_page_eos(&og_) != 0)
                     break;
             }
         }
@@ -142,25 +149,25 @@ void OggEncoder::encode(const msg::PcmChunk* chunk)
     if (res > 0)
     {
         res /= sampleFormat_.msRate();
-        // LOG(INFO) << "res: " << res << "\n";
+        // LOG(INFO, LOG_TAG) << "res: " << res << "\n";
         lastGranulepos_ = os_.granulepos;
         // make oggChunk smaller
-        oggChunk->payload = (char*)realloc(oggChunk->payload, pos);
+        oggChunk->payload = static_cast<char*>(realloc(oggChunk->payload, pos));
         oggChunk->payloadSize = pos;
-        listener_->onChunkEncoded(this, oggChunk, res);
+        encoded_callback_(*this, oggChunk, res);
     }
 }
 
 
 void OggEncoder::initEncoder()
 {
-    if (codecOptions_.find(":") == string::npos)
+    if (codecOptions_.find(':') == string::npos)
         throw SnapException("Invalid codec options: \"" + codecOptions_ + "\"");
-    string mode = utils::string::trim_copy(codecOptions_.substr(0, codecOptions_.find(":")));
+    string mode = utils::string::trim_copy(codecOptions_.substr(0, codecOptions_.find(':')));
     if (mode != "VBR")
-        throw SnapException("Unsupported codec mode: \"" + mode + "\". Available: \"VBR\"");
+        throw SnapException("Unsupported codec mode: \"" + mode + R"(". Available: "VBR")");
 
-    string qual = utils::string::trim_copy(codecOptions_.substr(codecOptions_.find(":") + 1));
+    string qual = utils::string::trim_copy(codecOptions_.substr(codecOptions_.find(':') + 1));
     double quality = 1.0;
     try
     {
@@ -175,6 +182,8 @@ void OggEncoder::initEncoder()
         throw SnapException("compression level has to be between -0.1 and 1.0");
     }
 
+    LOG(INFO, LOG_TAG) << "Init - quality: " << quality << "\n";
+
     /********** Encode setup ************/
     vorbis_info_init(&vi_);
 
@@ -213,14 +222,14 @@ void OggEncoder::initEncoder()
      mode that libVorbis does not support (eg, too low a bitrate, etc,
      will return 'OV_EIMPL') */
 
-    if (ret)
+    if (ret != 0)
         throw SnapException("failed to init encoder");
 
     /* add a comment */
     vorbis_comment_init(&vc_);
     vorbis_comment_add_tag(&vc_, "TITLE", "SnapStream");
     vorbis_comment_add_tag(&vc_, "VERSION", VERSION);
-    vorbis_comment_add_tag(&vc_, "SAMPLE_FORMAT", sampleFormat_.getFormat().c_str());
+    vorbis_comment_add_tag(&vc_, "SAMPLE_FORMAT", sampleFormat_.toString().c_str());
 
     /* set up the analysis state and auxiliary encoding storage */
     vorbis_analysis_init(&vd_, &vi_);
@@ -259,8 +268,8 @@ void OggEncoder::initEncoder()
         if (result == 0)
             break;
         headerChunk_->payloadSize += og_.header_len + og_.body_len;
-        headerChunk_->payload = (char*)realloc(headerChunk_->payload, headerChunk_->payloadSize);
-        LOG(DEBUG) << "HeadLen: " << og_.header_len << ", bodyLen: " << og_.body_len << ", result: " << result << "\n";
+        headerChunk_->payload = static_cast<char*>(realloc(headerChunk_->payload, headerChunk_->payloadSize));
+        LOG(DEBUG, LOG_TAG) << "HeadLen: " << og_.header_len << ", bodyLen: " << og_.body_len << ", result: " << result << "\n";
         memcpy(headerChunk_->payload + pos, og_.header, og_.header_len);
         pos += og_.header_len;
         memcpy(headerChunk_->payload + pos, og_.body, og_.body_len);
