@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,32 +16,37 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#include <chrono>
-#include <memory>
-#include <sys/resource.h>
-
+// local headers
 #include "common/popl.hpp"
 #ifdef HAS_DAEMON
 #include "common/daemon.hpp"
 #endif
-#include "common/sample_format.hpp"
 #include "common/snap_exception.hpp"
-#include "common/time_defs.hpp"
 #include "common/utils/string_utils.hpp"
+#include "common/version.hpp"
 #include "encoder/encoder_factory.hpp"
-#include "message/message.hpp"
+#include "server.hpp"
 #include "server_settings.hpp"
-#include "stream_server.hpp"
 #if defined(HAS_AVAHI) || defined(HAS_BONJOUR)
 #include "publishZeroConf/publish_mdns.hpp"
 #endif
 #include "common/aixlog.hpp"
 #include "config.hpp"
 
+// 3rd party headers
+#include <boost/asio/ip/host_name.hpp>
+
+// standard headers
+#include <chrono>
+#include <memory>
+#include <sys/resource.h>
+
 
 using namespace std;
 using namespace popl;
 
+static constexpr auto LOG_TAG = "Snapserver";
+
 
 int main(int argc, char* argv[])
 {
@@ -53,7 +58,7 @@ int main(int argc, char* argv[])
     try
     {
         ServerSettings settings;
-        std::string pcmStream = "pipe:///tmp/snapfifo?name=default";
+        std::string pcmSource = "pipe:///tmp/snapfifo?name=default";
         std::string config_file = "/etc/snapserver.conf";
 
         OptionParser op("Allowed options");
@@ -63,54 +68,63 @@ int main(int argc, char* argv[])
 #ifdef HAS_DAEMON
         int processPriority(0);
         auto daemonOption = op.add<Implicit<int>>("d", "daemon", "Daemonize\noptional process priority [-20..19]", 0, &processPriority);
-        auto userValue = op.add<Value<string>>("", "user", "the user[:group] to run snapserver as when daemonized", "");
 #endif
-
         op.add<Value<string>>("c", "config", "path to the configuration file", config_file, &config_file);
 
-        // debug settings
-        OptionParser conf("");
-        conf.add<Value<bool>>("", "logging.debug", "enable debug logging", settings.logging.debug, &settings.logging.debug);
-        conf.add<Value<string>>("", "logging.debug_logfile", "log file name for the debug logs (debug must be enabled)", settings.logging.debug_logfile,
-                                &settings.logging.debug_logfile);
+        OptionParser conf("Overridable config file options");
+
+        // server settings
+        conf.add<Value<int>>("", "server.threads", "number of server threads", settings.server.threads, &settings.server.threads);
+        conf.add<Value<string>>("", "server.pidfile", "pid file when running as daemon", settings.server.pid_file, &settings.server.pid_file);
+        conf.add<Value<string>>("", "server.user", "the user to run as when daemonized", settings.server.user, &settings.server.user);
+        conf.add<Implicit<string>>("", "server.group", "the group to run as when daemonized", settings.server.group, &settings.server.group);
+        conf.add<Implicit<string>>("", "server.datadir", "directory where persistent data is stored", settings.server.data_dir, &settings.server.data_dir);
+
+        // HTTP RPC settings
+        conf.add<Value<bool>>("", "http.enabled", "enable HTTP Json RPC (HTTP POST and websockets)", settings.http.enabled, &settings.http.enabled);
+        conf.add<Value<size_t>>("", "http.port", "which port the server should listen on", settings.http.port, &settings.http.port);
+        auto http_bind_to_address = conf.add<Value<string>>("", "http.bind_to_address", "address for the server to listen on",
+                                                            settings.http.bind_to_address.front(), &settings.http.bind_to_address[0]);
+        conf.add<Implicit<string>>("", "http.doc_root", "serve a website from the doc_root location", settings.http.doc_root, &settings.http.doc_root);
+        conf.add<Value<string>>("", "http.host", "Hostname or IP under which clients can reach this host", settings.http.host, &settings.http.host);
+
+        // TCP RPC settings
+        conf.add<Value<bool>>("", "tcp.enabled", "enable TCP Json RPC)", settings.tcp.enabled, &settings.tcp.enabled);
+        conf.add<Value<size_t>>("", "tcp.port", "which port the server should listen on", settings.tcp.port, &settings.tcp.port);
+        auto tcp_bind_to_address = conf.add<Value<string>>("", "tcp.bind_to_address", "address for the server to listen on",
+                                                           settings.tcp.bind_to_address.front(), &settings.tcp.bind_to_address[0]);
 
         // stream settings
-        conf.add<Value<size_t>>("", "stream.port", "Server port", settings.stream.port, &settings.stream.port);
-        auto streamValue = conf.add<Value<string>>(
-            "", "stream.stream", "URI of the PCM input stream.\nFormat: TYPE://host/path?name=NAME\n[&codec=CODEC]\n[&sampleformat=SAMPLEFORMAT]", pcmStream,
-            &pcmStream);
-        int num_threads = -1;
-        conf.add<Value<int>>("", "server.threads", "number of server threads", num_threads, &num_threads);
-        std::string pid_file = "/var/run/snapserver/pid";
-        conf.add<Value<string>>("", "server.pidfile", "pid file when running as daemon", pid_file, &pid_file);
-        std::string data_dir;
-        conf.add<Implicit<string>>("", "server.datadir", "directory where persistent data is stored", data_dir, &data_dir);
+        auto stream_bind_to_address = conf.add<Value<string>>("", "stream.bind_to_address", "address for the server to listen on",
+                                                              settings.stream.bind_to_address.front(), &settings.stream.bind_to_address[0]);
+        conf.add<Value<size_t>>("", "stream.port", "which port the server should listen on", settings.stream.port, &settings.stream.port);
+        // deprecated: stream.stream, use stream.source instead
+        auto streamValue = conf.add<Value<string>>("", "stream.stream", "Deprecated: use stream.source", pcmSource, &pcmSource);
+        auto sourceValue = conf.add<Value<string>>(
+            "", "stream.source", "URI of the PCM input stream.\nFormat: TYPE://host/path?name=NAME\n[&codec=CODEC]\n[&sampleformat=SAMPLEFORMAT]", pcmSource,
+            &pcmSource);
 
         conf.add<Value<string>>("", "stream.sampleformat", "Default sample format", settings.stream.sampleFormat, &settings.stream.sampleFormat);
         conf.add<Value<string>>("", "stream.codec", "Default transport codec\n(flac|ogg|opus|pcm)[:options]\nType codec:? to get codec specific options",
                                 settings.stream.codec, &settings.stream.codec);
         // deprecated: stream_buffer, use chunk_ms instead
-        conf.add<Value<size_t>>("", "stream.stream_buffer", "Default stream read chunk size [ms]", settings.stream.streamChunkMs,
-                                &settings.stream.streamChunkMs);
+        conf.add<Value<size_t>>("", "stream.stream_buffer", "Default stream read chunk size [ms], deprecated, use stream.chunk_ms instead",
+                                settings.stream.streamChunkMs, &settings.stream.streamChunkMs);
         conf.add<Value<size_t>>("", "stream.chunk_ms", "Default stream read chunk size [ms]", settings.stream.streamChunkMs, &settings.stream.streamChunkMs);
         conf.add<Value<int>>("", "stream.buffer", "Buffer [ms]", settings.stream.bufferMs, &settings.stream.bufferMs);
         conf.add<Value<bool>>("", "stream.send_to_muted", "Send audio to muted clients", settings.stream.sendAudioToMutedClients,
                               &settings.stream.sendAudioToMutedClients);
-        auto stream_bind_to_address = conf.add<Value<string>>("", "stream.bind_to_address", "address for the server to listen on",
-                                                              settings.stream.bind_to_address.front(), &settings.stream.bind_to_address[0]);
 
-        // HTTP RPC settings
-        conf.add<Value<bool>>("", "http.enabled", "enable HTTP Json RPC (HTTP POST and websockets)", settings.http.enabled, &settings.http.enabled);
-        conf.add<Value<size_t>>("", "http.port", "which port the server should listen to", settings.http.port, &settings.http.port);
-        auto http_bind_to_address = conf.add<Value<string>>("", "http.bind_to_address", "address for the server to listen on",
-                                                            settings.http.bind_to_address.front(), &settings.http.bind_to_address[0]);
-        conf.add<Implicit<string>>("", "http.doc_root", "serve a website from the doc_root location", settings.http.doc_root, &settings.http.doc_root);
+        // streaming_client options
+        conf.add<Value<uint16_t>>("", "streaming_client.initial_volume", "Volume [percent] assigned to new streaming clients",
+                                  settings.streamingclient.initialVolume, &settings.streamingclient.initialVolume);
 
-        // TCP RPC settings
-        conf.add<Value<bool>>("", "tcp.enabled", "enable TCP Json RPC)", settings.tcp.enabled, &settings.tcp.enabled);
-        conf.add<Value<size_t>>("", "tcp.port", "which port the server should listen to", settings.tcp.port, &settings.tcp.port);
-        auto tcp_bind_to_address = conf.add<Value<string>>("", "tcp.bind_to_address", "address for the server to listen on",
-                                                           settings.tcp.bind_to_address.front(), &settings.tcp.bind_to_address[0]);
+        // logging settings
+        conf.add<Value<string>>("", "logging.sink", "log sink [null,system,stdout,stderr,file:<filename>]", settings.logging.sink, &settings.logging.sink);
+        auto logfilterOption = conf.add<Value<string>>(
+            "", "logging.filter",
+            "log filter <tag>:<level>[,<tag>:<level>]* with tag = * or <log tag> and level = [trace,debug,info,notice,warning,error,fatal]",
+            settings.logging.filter);
 
         try
         {
@@ -138,19 +152,19 @@ int main(int argc, char* argv[])
         }
         catch (const std::invalid_argument& e)
         {
-            SLOG(ERROR) << "Exception: " << e.what() << std::endl;
+            cerr << "Exception: " << e.what() << std::endl;
             cout << "\n" << op << "\n";
             exit(EXIT_FAILURE);
         }
 
         if (versionSwitch->is_set())
         {
-            cout << "snapserver v" << VERSION << "\n"
-                 << "Copyright (C) 2014-2020 BadAix (snapcast@badaix.de).\n"
+            cout << "snapserver v" << version::code << (!version::rev().empty() ? (" (rev " + version::rev(8) + ")") : ("")) << "\n"
+                 << "Copyright (C) 2014-2022 BadAix (snapcast@badaix.de).\n"
                  << "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n"
                  << "This is free software: you are free to change and redistribute it.\n"
                  << "There is NO WARRANTY, to the extent permitted by law.\n\n"
-                 << "Written by Johannes Pohl.\n";
+                 << "Written by Johannes M. Pohl and contributors <https://github.com/badaix/snapcast/graphs/contributors>.\n\n";
             exit(EXIT_SUCCESS);
         }
 
@@ -182,66 +196,82 @@ int main(int argc, char* argv[])
             exit(EXIT_SUCCESS);
         }
 
-        AixLog::Log::init<AixLog::SinkNative>("snapserver", AixLog::Severity::trace, AixLog::Type::special);
-        if (settings.logging.debug)
+        settings.logging.filter = logfilterOption->value();
+        if (logfilterOption->is_set())
         {
-            AixLog::Log::instance().add_logsink<AixLog::SinkCout>(AixLog::Severity::trace, AixLog::Type::all, "%Y-%m-%d %H-%M-%S.#ms [#severity] (#tag_func)");
-            if (!settings.logging.debug_logfile.empty())
-                AixLog::Log::instance().add_logsink<AixLog::SinkFile>(AixLog::Severity::trace, AixLog::Type::all, settings.logging.debug_logfile,
-                                                                      "%Y-%m-%d %H-%M-%S.#ms [#severity] (#tag_func)");
+            for (size_t n = 1; n < logfilterOption->count(); ++n)
+                settings.logging.filter += "," + logfilterOption->value(n);
         }
+
+        if (settings.logging.sink.empty())
+        {
+            settings.logging.sink = "stdout";
+#ifdef HAS_DAEMON
+            if (daemonOption->is_set())
+                settings.logging.sink = "system";
+#endif
+        }
+        AixLog::Filter logfilter;
+        auto filters = utils::string::split(settings.logging.filter, ',');
+        for (const auto& filter : filters)
+            logfilter.add_filter(filter);
+
+        string logformat = "%Y-%m-%d %H-%M-%S.#ms [#severity] (#tag_func)";
+        if (settings.logging.sink.find("file:") != string::npos)
+        {
+            string logfile = settings.logging.sink.substr(settings.logging.sink.find(':') + 1);
+            AixLog::Log::init<AixLog::SinkFile>(logfilter, logfile, logformat);
+        }
+        else if (settings.logging.sink == "stdout")
+            AixLog::Log::init<AixLog::SinkCout>(logfilter, logformat);
+        else if (settings.logging.sink == "stderr")
+            AixLog::Log::init<AixLog::SinkCerr>(logfilter, logformat);
+        else if (settings.logging.sink == "system")
+            AixLog::Log::init<AixLog::SinkNative>("snapserver", logfilter);
+        else if (settings.logging.sink == "null")
+            AixLog::Log::init<AixLog::SinkNull>();
         else
-        {
-            AixLog::Log::instance().add_logsink<AixLog::SinkCout>(AixLog::Severity::info, AixLog::Type::all, "%Y-%m-%d %H-%M-%S [#severity] (#tag_func)");
-        }
+            throw SnapException("Invalid log sink: " + settings.logging.sink);
 
-        for (const auto& opt : conf.unknown_options())
-            LOG(WARNING) << "unknown configuration option: " << opt << "\n";
+        LOG(INFO, LOG_TAG) << "Version " << version::code << (!version::rev().empty() ? (", revision " + version::rev(8)) : ("")) << "\n";
 
-        if (!streamValue->is_set())
-            settings.stream.pcmStreams.push_back(streamValue->value());
+        if (!streamValue->is_set() && !sourceValue->is_set())
+            settings.stream.sources.push_back(sourceValue->value());
 
         for (size_t n = 0; n < streamValue->count(); ++n)
         {
-            LOG(INFO) << "Adding stream: " << streamValue->value(n) << "\n";
-            settings.stream.pcmStreams.push_back(streamValue->value(n));
+            LOG(INFO, LOG_TAG) << "Adding stream: " << streamValue->value(n) << "\n";
+            settings.stream.sources.push_back(streamValue->value(n));
+        }
+        for (size_t n = 0; n < sourceValue->count(); ++n)
+        {
+            LOG(INFO, LOG_TAG) << "Adding source: " << sourceValue->value(n) << "\n";
+            settings.stream.sources.push_back(sourceValue->value(n));
         }
 
 #ifdef HAS_DAEMON
         std::unique_ptr<Daemon> daemon;
         if (daemonOption->is_set())
         {
-            string user = "";
-            string group = "";
+            if (settings.server.user.empty())
+                throw std::invalid_argument("user must not be empty");
 
-            if (userValue->is_set())
-            {
-                if (userValue->value().empty())
-                    std::invalid_argument("user must not be empty");
+            if (settings.server.data_dir.empty())
+                settings.server.data_dir = "/var/lib/snapserver";
+            Config::instance().init(settings.server.data_dir, settings.server.user, settings.server.group);
 
-                vector<string> user_group = utils::string::split(userValue->value(), ':');
-                user = user_group[0];
-                if (user_group.size() > 1)
-                    group = user_group[1];
-            }
-            if (data_dir.empty())
-                data_dir = "/var/lib/snapserver";
-            Config::instance().init(data_dir, user, group);
-            daemon.reset(new Daemon(user, group, pid_file));
-            SLOG(NOTICE) << "daemonizing" << std::endl;
-            daemon->daemonize();
-            if (processPriority < -20)
-                processPriority = -20;
-            else if (processPriority > 19)
-                processPriority = 19;
+            daemon = std::make_unique<Daemon>(settings.server.user, settings.server.group, settings.server.pid_file);
+            processPriority = std::min(std::max(-20, processPriority), 19);
             if (processPriority != 0)
                 setpriority(PRIO_PROCESS, 0, processPriority);
-            SLOG(NOTICE) << "daemon started" << std::endl;
+            LOG(NOTICE, LOG_TAG) << "daemonizing" << std::endl;
+            daemon->daemonize();
+            LOG(NOTICE, LOG_TAG) << "daemon started" << std::endl;
         }
         else
-            Config::instance().init(data_dir);
+            Config::instance().init(settings.server.data_dir);
 #else
-        Config::instance().init();
+        Config::instance().init(settings.server.data_dir);
 #endif
 
         boost::asio::io_context io_context;
@@ -261,37 +291,49 @@ int main(int argc, char* argv[])
         }
         publishZeroConfg->publish(dns_services);
 #endif
+        if (settings.http.enabled)
+        {
+            if ((settings.http.host == "<hostname>") || settings.http.host.empty())
+            {
+                settings.http.host = boost::asio::ip::host_name();
+                LOG(INFO, LOG_TAG) << "Using HTTP host name: " << settings.http.host << "\n";
+            }
+        }
+
         if (settings.stream.streamChunkMs < 10)
         {
-            LOG(WARNING) << "Stream read chunk size is less than 10ms, changing to 10ms\n";
+            LOG(WARNING, LOG_TAG) << "Stream read chunk size is less than 10ms, changing to 10ms\n";
             settings.stream.streamChunkMs = 10;
         }
 
-        if (settings.stream.bufferMs < 400)
+        static constexpr chrono::milliseconds MIN_BUFFER_DURATION = 20ms;
+        if (settings.stream.bufferMs < MIN_BUFFER_DURATION.count())
         {
-            LOG(WARNING) << "Buffer is less than 400ms, changing to 400ms\n";
-            settings.stream.bufferMs = 400;
+            LOG(WARNING, LOG_TAG) << "Buffer is less than " << MIN_BUFFER_DURATION.count() << "ms, changing to " << MIN_BUFFER_DURATION.count() << "ms\n";
+            settings.stream.bufferMs = MIN_BUFFER_DURATION.count();
         }
 
-        auto streamServer = std::make_unique<StreamServer>(io_context, settings);
-        streamServer->start();
+        auto server = std::make_unique<Server>(io_context, settings);
+        server->start();
 
-        if (num_threads < 0)
-            num_threads = std::max(2, std::min(4, static_cast<int>(std::thread::hardware_concurrency())));
-        LOG(INFO) << "number of threads: " << num_threads << ", hw threads: " << std::thread::hardware_concurrency() << "\n";
+        if (settings.server.threads < 0)
+            settings.server.threads = std::max(2, std::min(4, static_cast<int>(std::thread::hardware_concurrency())));
+        LOG(INFO, LOG_TAG) << "Number of threads: " << settings.server.threads << ", hw threads: " << std::thread::hardware_concurrency() << "\n";
 
         // Construct a signal set registered for process termination.
         boost::asio::signal_set signals(io_context, SIGHUP, SIGINT, SIGTERM);
-        signals.async_wait([&io_context](const boost::system::error_code& ec, int signal) {
+        signals.async_wait(
+            [&io_context](const boost::system::error_code& ec, int signal)
+            {
             if (!ec)
-                SLOG(INFO) << "Received signal " << signal << ": " << strsignal(signal) << "\n";
+                LOG(INFO, LOG_TAG) << "Received signal " << signal << ": " << strsignal(signal) << "\n";
             else
-                SLOG(INFO) << "Failed to wait for signal: " << ec << "\n";
+                LOG(INFO, LOG_TAG) << "Failed to wait for signal, error: " << ec.message() << "\n";
             io_context.stop();
         });
 
         std::vector<std::thread> threads;
-        for (int n = 0; n < num_threads; ++n)
+        for (int n = 0; n < settings.server.threads; ++n)
             threads.emplace_back([&] { io_context.run(); });
 
         io_context.run();
@@ -299,16 +341,16 @@ int main(int argc, char* argv[])
         for (auto& t : threads)
             t.join();
 
-        LOG(INFO) << "Stopping streamServer" << endl;
-        streamServer->stop();
-        LOG(INFO) << "done" << endl;
+        LOG(INFO, LOG_TAG) << "Stopping streamServer" << endl;
+        server->stop();
+        LOG(INFO, LOG_TAG) << "done" << endl;
     }
     catch (const std::exception& e)
     {
-        SLOG(ERROR) << "Exception: " << e.what() << std::endl;
+        LOG(ERROR, LOG_TAG) << "Exception: " << e.what() << std::endl;
         exitcode = EXIT_FAILURE;
     }
     Config::instance().save();
-    SLOG(NOTICE) << "daemon terminated." << endl;
+    LOG(NOTICE, LOG_TAG) << "Snapserver terminated." << endl;
     exit(exitcode);
 }
