@@ -1,6 +1,6 @@
 /***
     This file is part of snapcast
-    Copyright (C) 2014-2020  Johannes Pohl
+    Copyright (C) 2014-2024  Johannes Pohl
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,31 +16,33 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***/
 
-#ifndef STREAM_SESSION_HPP
-#define STREAM_SESSION_HPP
+#pragma once
 
-#include "common/queue.h"
-#include "message/message.hpp"
+
+// local headers
+#include "common/message/message.hpp"
 #include "streamreader/stream_manager.hpp"
-#include <atomic>
-#include <boost/asio.hpp>
-#include <condition_variable>
+
+// 3rd party headers
+#include <boost/asio/any_io_executor.hpp>
+#include <boost/asio/buffer.hpp>
+#include <boost/asio/strand.hpp>
+
+// standard headers
 #include <deque>
 #include <memory>
-#include <set>
+#include <mutex>
 #include <sstream>
 #include <string>
 #include <vector>
 
 
-using boost::asio::ip::tcp;
-
 
 class StreamSession;
 
 
 /// Interface: callback for a received message.
-class MessageReceiver
+class StreamMessageReceiver
 {
 public:
     virtual void onMessageReceived(StreamSession* connection, const msg::BaseMessage& baseMessage, char* buffer) = 0;
@@ -55,6 +57,7 @@ class shared_const_buffer
     {
         std::vector<char> data;
         bool is_pcm_chunk;
+        message_type type;
         chronos::time_point_clk rec_time;
     };
 
@@ -65,6 +68,7 @@ public:
         message.sent = t;
         const msg::PcmChunk* pcm_chunk = dynamic_cast<const msg::PcmChunk*>(&message);
         message_ = std::make_shared<Message>();
+        message_->type = message.type;
         message_->is_pcm_chunk = (pcm_chunk != nullptr);
         if (message_->is_pcm_chunk)
             message_->rec_time = pcm_chunk->start();
@@ -77,8 +81,8 @@ public:
     }
 
     // Implement the ConstBufferSequence requirements.
-    typedef boost::asio::const_buffer value_type;
-    typedef const boost::asio::const_buffer* const_iterator;
+    using value_type = boost::asio::const_buffer;
+    using const_iterator = const boost::asio::const_buffer*;
     const boost::asio::const_buffer* begin() const
     {
         return &buffer_;
@@ -102,55 +106,59 @@ private:
 };
 
 
+using WriteHandler = std::function<void(boost::system::error_code ec, std::size_t length)>;
+
 /// Endpoint for a connected client.
 /**
  * Endpoint for a connected client.
  * Messages are sent to the client with the "send" method.
- * Received messages from the client are passed to the MessageReceiver callback
+ * Received messages from the client are passed to the StreamMessageReceiver callback
  */
 class StreamSession : public std::enable_shared_from_this<StreamSession>
 {
 public:
-    /// ctor. Received message from the client are passed to MessageReceiver
-    StreamSession(boost::asio::io_context& ioc, MessageReceiver* receiver, tcp::socket&& socket);
-    ~StreamSession();
-    void start();
-    void stop();
+    /// ctor. Received message from the client are passed to StreamMessageReceiver
+    StreamSession(const boost::asio::any_io_executor& executor, StreamMessageReceiver* receiver);
+    virtual ~StreamSession() = default;
+
+    virtual std::string getIP() = 0;
+
+    virtual void start() = 0;
+    virtual void stop() = 0;
+
+    void setMessageReceiver(StreamMessageReceiver* receiver)
+    {
+        messageReceiver_ = receiver;
+    }
+
+protected:
+    virtual void sendAsync(const shared_const_buffer& buffer, const WriteHandler& handler) = 0;
+
+public:
+    /// Sends a message to the client (asynchronous)
+    void send(msg::message_ptr message);
 
     /// Sends a message to the client (asynchronous)
-    void sendAsync(msg::message_ptr message, bool send_now = false);
-
-    /// Sends a message to the client (asynchronous)
-    void sendAsync(shared_const_buffer const_buf, bool send_now = false);
+    void send(shared_const_buffer const_buf);
 
     /// Max playout latency. No need to send PCM data that is older than bufferMs
     void setBufferMs(size_t bufferMs);
 
     std::string clientId;
 
-    std::string getIP()
-    {
-        return socket_.remote_endpoint().address().to_string();
-    }
-
     void setPcmStream(streamreader::PcmStreamPtr pcmStream);
     const streamreader::PcmStreamPtr pcmStream() const;
 
 protected:
-    void read_next();
     void send_next();
 
     msg::BaseMessage baseMessage_;
     std::vector<char> buffer_;
     size_t base_msg_size_;
-    tcp::socket socket_;
-    MessageReceiver* messageReceiver_;
+    StreamMessageReceiver* messageReceiver_;
     size_t bufferMs_;
     streamreader::PcmStreamPtr pcmStream_;
-    boost::asio::io_context::strand strand_;
+    boost::asio::strand<boost::asio::any_io_executor> strand_;
     std::deque<shared_const_buffer> messages_;
+    mutable std::mutex mutex_;
 };
-
-
-
-#endif
